'' =================================================================================================
''
''   File....... jm_ws2801_demo.spin
''   Purpose.... Demonstrate high-performance driver for the WS2801
''   Author..... Jon "JonnyMac" McPhalen 
''               Copyright (c) 2012 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 28 SEP 2012
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1     = 31                                                  ' programming / debug port
  TX1     = 30
  
  SDA     = 29                                                  ' eeprom
  SCL     = 28


  ' WS2801 connections
  ' -- requires 3.3v to 5v translation (e.g., TC4427, 74x245, etc.)
  
  CK      =  9
  SD      =  8


con

  CHANNELS = 7
  LAST_CH  = CHANNELS - 1


obj

  leds : "jm_ws2801"
 

var

  long  cbuf0[CHANNELS]                                         ' color buffers
  long  cbuf1[CHANNELS]                                         
  long  cbuf2[CHANNELS]

  long  stack1[16]                                              ' for background cogs
  long  stack2[16]
  

pub main | level, color, ch

  low(27)                                                       ' enable 74x245 on HC-8+
  leds.start(SD, CK, CHANNELS, 5, true)                         ' start WS2801 driver (swap rb)

  ' quick color demo

  repeat ch from 0 to LAST_CH
    leds.set(ch, leds#RED)
    pause(50)
  pause(500)
    
  repeat ch from 0 to LAST_CH
    leds.set(ch, leds#GREEN)
    pause(50)
  pause(500)
  
  repeat ch from 0 to LAST_CH
    leds.set(ch, leds#BLUE)
    pause(50)
  pause(500)

  repeat ch from 0 to LAST_CH
    leds.set(ch, leds#WHITE)
    pause(50)
  pause(500)

  repeat ch from 0 to LAST_CH
    leds.set(ch, $00_00_00)
    pause(50)
  pause(1000)

  
  ' lauch animation cogs

  cognew(color_shift, @stack1)                                  ' start color shifter

  longfill(@cbuf2, -1, CHANNELS)                                ' set cbuf2 to "no color"   
  cognew(larsen_scan, @stack2)                                  ' start larsen scanner


  ' mix colors
  ' -- priority to cbuf2

  repeat
    longmove(@cbuf0, @cbuf1, CHANNELS)                          ' copy background color (cbuf1)     
    repeat ch from 0 to LAST_CH                                 ' loop through array
      if (cbuf2[ch] => 0)                                       '  if color in cbuf2
        cbuf0[ch] := cbuf2[ch]                                  '    use it

    longmove(leds.address, @cbuf0, CHANNELS)                    ' update driver


pri color_shift | level, color

'' Color shift all leds from red to blue and back
'' -- color written to cbuf1 for mixing by foreground cog

  ' fade from off to deep red

  repeat level from $00 to $1F                                  ' from off to deep red
    longfill(@cbuf1, leds.make_rgb(level, 0, 0), CHANNELS)
    pause(65)  

  repeat

    ' red to blue
    
    repeat level from $1F to $00            
      longfill(@cbuf1, leds.make_rgb(level, 0, $1F-level), CHANNELS)
      pause(65)

    ' blue to red 

    repeat level from $00 to $1F            
      longfill(@cbuf1, leds.make_rgb(level, 0, $1F-level), CHANNELS)   
      pause(65)
      

pri larsen_scan | ch

'' Create Larsen scanner (Cylon, Knight Rider)
'' -- color written to cbuf2 for mixing by foreground cog

  repeat
    repeat ch from 0 to LAST_CH-1                               ' loop forward
      longfill(@cbuf2, -1, CHANNELS)                            ' remove color
      cbuf2[ch] := $1F_1F_1F                                    ' set to white
      pause(1000 / CHANNELS)                                    ' hold

    repeat ch from LAST_CH to 1                                 ' loop back
      longfill(@cbuf2, -1, CHANNELS)
      cbuf2[ch] := $1F_1F_1F
      pause(1000 / CHANNELS)
      
  
con

  { ------------- }
  {  B A S I C S  } 
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1792                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]

          
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  